#ifndef DEFER_BLINN_PHONG_LOCAL_H
#define DEFER_BLINN_PHONG_LOCAL_H
/* Singleton and derived class of State pattern. Class for deferred shading with Blinn-Phong illumination and
   point lights with local influence */


#include "DeferShader.h"


class DeferBlinnPhongLocal: public DeferShader {
    public:
        //Get a pointer to the DeferBlinnPhongLocal singleton (and construct if this hasn't been done yet)
        static DeferBlinnPhongLocal* getInstance(Geometry* geoLights);

        //Get a pointer to the DeferBlinnPhongLocal singleton only (no construction happens)
        static DeferBlinnPhongLocal* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static DeferBlinnPhongLocal* singleton;


    private:
        //Constructor
        DeferBlinnPhongLocal(Geometry* geoLights, std::string gp_fileName, std::string fp_fileName, std::string stateInfo_): DeferShader(geoLights, gp_fileName, fp_fileName, stateInfo_) {}
};

#endif
