#ifndef DEFER_COOK_TORRANCE_GLOBAL_H
#define DEFER_COOK_TORRANCE_GLOBAL_H
/* Singleton and derived class of State pattern. Class for deferred shading with Cook-Torrance illumination and
   point lights with global influence */


#include "DeferShader.h"


class DeferCookTorranceGlobal: public DeferShader {
    public:
        //Get a pointer to the DeferCookTorranceGlobal singleton (and construct if this hasn't been done yet)
        static DeferCookTorranceGlobal* getInstance(Geometry* geoLights);

        //Get a pointer to the DeferCookTorranceGlobal singleton only (no construction happens)
        static DeferCookTorranceGlobal* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static DeferCookTorranceGlobal* singleton;


    private:
        //Constructor
        DeferCookTorranceGlobal(Geometry* geoLights, std::string gp_fileName, std::string fp_fileName, std::string stateInfo_): DeferShader(geoLights, gp_fileName, fp_fileName, stateInfo_) {}
};

#endif
