#ifndef DEFER_COOK_TORRANCE_SPOT_H
#define DEFER_COOK_TORRANCE_SPOT_H
/* Singleton and derived class of State pattern. Class for deferred shading with Cook-Torrance illumination and
   spot lights */


#include "DeferShader.h"


class DeferCookTorranceSpot: public DeferShader {
    public:
        //Get a pointer to the DeferCookTorranceSpot singleton (and construct if this hasn't been done yet)
        static DeferCookTorranceSpot* getInstance(Geometry* geoLights);

        //Get a pointer to the DeferCookTorranceSpot singleton only (no construction happens)
        static DeferCookTorranceSpot* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static DeferCookTorranceSpot* singleton;


    private:
        //Constructor
        DeferCookTorranceSpot(Geometry* geoLights, std::string gp_fileName, std::string fp_fileName, std::string stateInfo_): DeferShader(geoLights, gp_fileName, fp_fileName, stateInfo_) {}
};

#endif
