#include <cstdlib>
#include "Geometry.h"
#include "Window.h"
#include "Fbo.h"
#include "FboManager.h"
#include "FullScreenQuad.h"
#include "TradGoochGlobal.h"
#include "DeferGoochLocal.h"
#include "DeferGoochSpot.h"
#include "DeferPhongGlobal.h"
#include "DeferBlinnPhongGlobal.h"
#include "DeferCookTorranceGlobal.h"
#include "DeferGoochGlobal.h"


DeferGoochGlobal* DeferGoochGlobal::singleton = NULL;


//Constructor
DeferGoochGlobal::DeferGoochGlobal(Geometry* geoLights, std::string gp_fileName, std::string fp_fileName, std::string stateInfo_): DeferShader(geoLights, gp_fileName, fp_fileName, stateInfo_) {
    //Gooch has its own ambient shading, so overwrite fp_Ambient
    fp_Ambient               = shaderProgFactory->loadProgram("Source/ShaderPrograms/DeferredShading/fp_AmbientGooch.cg", "main", FRAGMENT_PROGRAM);
    cg_TexSceneColor_Ambient = cgGetNamedParameter(fp_Ambient, "texSceneColor");
    cgGLSetTextureParameter(cg_TexSceneColor_Ambient, texSceneColor);
    //Setup Gooch 'post-processing'
    fboGooch    = fboManager->createFbo(1, 1, window->getWidth(), window->getHeight(), GL_TEXTURE_RECTANGLE_ARB, GL_NEAREST, GL_NEAREST, GL_RGBA16F_ARB, GL_RGBA);
    fp_Tex      = shaderProgFactory->loadProgram("Source/ShaderPrograms/DeferredShading/fp_Tex.cg", "main", FRAGMENT_PROGRAM);
    texGooch    = fboGooch->getTex(0);
    cg_TexGooch = cgGetNamedParameter(fp_Tex, "tex");
    cgGLSetTextureParameter(cg_TexGooch, texGooch);
}


//getInstance
DeferGoochGlobal* DeferGoochGlobal::getInstance(Geometry* geoLights) {
    if (singleton == NULL)
        singleton = new DeferGoochGlobal(geoLights, "Source/ShaderPrograms/DeferredShading/gp_FullScreenQuad.cg", "Source/ShaderPrograms/DeferredShading/fp_GoochGlobal.cg", "Deferred shading with Gooch illumination and point lights with global influence");
    return singleton;
}


//getPointer
DeferGoochGlobal* DeferGoochGlobal::getPointer() {
    return singleton;
}


//deactivate
void DeferGoochGlobal::deactivate() {
    //Deactivate rendering to G-Buffer
    cgGLDisableProfile(vertexProfile); cgGLDisableProfile(fragmentProfile);
    fbo->unbind();

    //Additive blending clamps colors to [0,1]. Prevent this by first rendering to an FBO and then draw the FBO's content
    //Activate rendering to texGooch
    fboGooch->bind(1);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  //Clear FBO

    //Perform deferred shading
    glDepthMask(GL_FALSE);

    //Draw ambient shaded scene
    drawAmbientScene();
    cgGLDisableProfile(fragmentProfile);
    //Enable additive blending and draw diffuse and specular scene (which is then added to the ambient shaded scene)
    glEnable(GL_BLEND);
    cgGLEnableTextureParameter(cg_TexSceneColor); cgGLEnableTextureParameter(cg_TexScenePos); cgGLEnableTextureParameter(cg_TexSceneNormal);
    cgGLBindProgram(gp_LightVolume); cgGLBindProgram(fp_DeferShader);  //vp_NoTransform is still bound
    cgGLEnableProfile(geometryProfile); cgGLEnableProfile(fragmentProfile);
    //Draw lights
    geoLights->draw(0, numLights);
    //Disable additive blending and shading
    cgGLDisableProfile(geometryProfile); cgGLDisableProfile(fragmentProfile);
    cgGLDisableTextureParameter(cg_TexSceneColor); cgGLDisableTextureParameter(cg_TexScenePos); cgGLDisableTextureParameter(cg_TexSceneNormal);
    glDisable(GL_BLEND);

    //Deactivate rendering to texGooch
    fboGooch->unbind();

    //Draw the Gooch shading by texturing texGooch to a full-screen quad
    cgGLEnableTextureParameter(cg_TexGooch);
    cgGLBindProgram(fp_Tex);  //vp_NoTransform is still bound
    cgGLEnableProfile(fragmentProfile);
    fullScreenQuad->draw();
    cgGLDisableProfile(vertexProfile); cgGLDisableProfile(fragmentProfile);
    cgGLDisableTextureParameter(cg_TexGooch);

    glDepthMask(GL_TRUE);
}


//ShadingState functions for switching between states
ShadingState* DeferGoochGlobal::enableTraditional() {
    return TradGoochGlobal::getPointer();
}
ShadingState* DeferGoochGlobal::enableDeferred() {
    return singleton;
}
ShadingState* DeferGoochGlobal::enableLocal() {
    return DeferGoochLocal::getPointer();
}
ShadingState* DeferGoochGlobal::enableSpot() {
    return DeferGoochSpot::getPointer();
}
ShadingState* DeferGoochGlobal::enableGlobal() {
    return singleton;
}
ShadingState* DeferGoochGlobal::enablePhong() {
    return DeferPhongGlobal::getPointer();
}
ShadingState* DeferGoochGlobal::enableBlinnPhong() {
    return DeferBlinnPhongGlobal::getPointer();
}
ShadingState* DeferGoochGlobal::enableCookTorrance() {
    return DeferCookTorranceGlobal::getPointer();
}
ShadingState* DeferGoochGlobal::enableGooch() {
    return singleton;
}
