#ifndef DEFER_GOOCH_GLOBAL_H
#define DEFER_GOOCH_GLOBAL_H
/* Singleton and derived class of State pattern. Class for deferred shading with Gooch illumination and
   point lights with global influence */


#include "DeferShader.h"


class DeferGoochGlobal: public DeferShader {
    public:
        //Get a pointer to the DeferGoochGlobal singleton (and construct if this hasn't been done yet)
        static DeferGoochGlobal* getInstance(Geometry* geoLights);

        //Get a pointer to the DeferGoochGlobal singleton only (no construction happens)
        static DeferGoochGlobal* getPointer();

        void deactivate();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static DeferGoochGlobal* singleton;
        //Gooch 'post-processing'
        Fbo* fboGooch;
        CGprogram fp_Tex;
        CGparameter cg_TexGooch;
        GLuint texGooch;


    private:
        //Constructor
        DeferGoochGlobal(Geometry* geoLights, std::string gp_fileName, std::string fp_fileName, std::string stateInfo_);
};

#endif
