#include <cstdlib>
#include "Geometry.h"
#include "Window.h"
#include "Fbo.h"
#include "FboManager.h"
#include "FullScreenQuad.h"
#include "TradGoochLocal.h"
#include "DeferGoochSpot.h"
#include "DeferGoochGlobal.h"
#include "DeferPhongLocal.h"
#include "DeferBlinnPhongLocal.h"
#include "DeferCookTorranceLocal.h"
#include "DeferGoochLocal.h"


DeferGoochLocal* DeferGoochLocal::singleton = NULL;


//Constructor
DeferGoochLocal::DeferGoochLocal(Geometry* geoLights, std::string gp_fileName, std::string fp_fileName, std::string stateInfo_): DeferShader(geoLights, gp_fileName, fp_fileName, stateInfo_) {
    //Gooch has its own ambient shading
    fp_Ambient               = shaderProgFactory->loadProgram("Source/ShaderPrograms/DeferredShading/fp_AmbientGooch.cg", "main", FRAGMENT_PROGRAM);
    cg_TexSceneColor_Ambient = cgGetNamedParameter(fp_Ambient, "texSceneColor");
    cgGLSetTextureParameter(cg_TexSceneColor_Ambient, texSceneColor);
    //Setup Gooch 'post-processing'
    fboGooch    = fboManager->createFbo(1, 1, window->getWidth(), window->getHeight(), GL_TEXTURE_RECTANGLE_ARB, GL_NEAREST, GL_NEAREST, GL_RGBA16F_ARB, GL_RGBA);
    fp_Tex      = shaderProgFactory->loadProgram("Source/ShaderPrograms/DeferredShading/fp_Tex.cg", "main", FRAGMENT_PROGRAM);
    texGooch    = fboGooch->getTex(0);
    cg_TexGooch = cgGetNamedParameter(fp_Tex, "tex");
    cgGLSetTextureParameter(cg_TexGooch, texGooch);
}


//getInstance
DeferGoochLocal* DeferGoochLocal::getInstance(Geometry* geoLights) {
    if (singleton == NULL)
        singleton = new DeferGoochLocal(geoLights, "Source/ShaderPrograms/DeferredShading/gp_Billboard.cg", "Source/ShaderPrograms/DeferredShading/fp_GoochLocal.cg", "Deferred shading with Gooch illumination and point lights with local influence");
    return singleton;
}


//getPointer
DeferGoochLocal* DeferGoochLocal::getPointer() {
    return singleton;
}


//deactivate
void DeferGoochLocal::deactivate() {
    //Deactivate rendering to G-Buffer
    cgGLDisableProfile(vertexProfile); cgGLDisableProfile(fragmentProfile);
    fbo->unbind();

    //Additive blending clamps colors to [0,1]. Prevent this by first render to an FBO and then draw the FBO's content
    //Activate rendering to texGooch
    fboGooch->bind(1);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  //Clear FBO

    //Perform deferred shading
    glDepthMask(GL_FALSE);

    //Draw ambient shaded scene
    drawAmbientScene();
    cgGLDisableProfile(fragmentProfile);
    //Enable additive blending and draw diffuse and specular scene (which is then added to the ambient shaded scene)
    glEnable(GL_BLEND);
    cgGLEnableTextureParameter(cg_TexSceneColor); cgGLEnableTextureParameter(cg_TexScenePos); cgGLEnableTextureParameter(cg_TexSceneNormal);
    cgGLBindProgram(gp_LightVolume); cgGLBindProgram(fp_DeferShader);  //vp_NoTransform is still bound
    cgGLEnableProfile(geometryProfile); cgGLEnableProfile(fragmentProfile);
    //Draw lights
    geoLights->draw(0, numLights);
    //Disable additive blending and shading
    cgGLDisableProfile(geometryProfile); cgGLDisableProfile(fragmentProfile);
    cgGLDisableTextureParameter(cg_TexSceneColor); cgGLDisableTextureParameter(cg_TexScenePos); cgGLDisableTextureParameter(cg_TexSceneNormal);
    glDisable(GL_BLEND);

    //Deactivate rendering to texGooch
    fboGooch->unbind();

    //Draw the Gooch shading by texturing texGooch to a full-screen quad
    cgGLEnableTextureParameter(cg_TexGooch);
    cgGLBindProgram(fp_Tex);  //vp_NoTransform is still bound
    cgGLEnableProfile(fragmentProfile);
    fullScreenQuad->draw();
    cgGLDisableProfile(vertexProfile); cgGLDisableProfile(fragmentProfile);
    cgGLDisableTextureParameter(cg_TexGooch);

    glDepthMask(GL_TRUE);
}


//ShadingState functions for switching between states
ShadingState* DeferGoochLocal::enableTraditional() {
    return TradGoochLocal::getPointer();
}
ShadingState* DeferGoochLocal::enableDeferred() {
    return singleton;
}
ShadingState* DeferGoochLocal::enableLocal() {
    return singleton;
}
ShadingState* DeferGoochLocal::enableSpot() {
    return DeferGoochSpot::getPointer();
}
ShadingState* DeferGoochLocal::enableGlobal() {
    return DeferGoochGlobal::getPointer();
}
ShadingState* DeferGoochLocal::enablePhong() {
    return DeferPhongLocal::getPointer();
}
ShadingState* DeferGoochLocal::enableBlinnPhong() {
    return DeferBlinnPhongLocal::getPointer();
}
ShadingState* DeferGoochLocal::enableCookTorrance() {
    return DeferCookTorranceLocal::getPointer();
}
ShadingState* DeferGoochLocal::enableGooch() {
    return singleton;
}
