#ifndef DEFER_GOOCH_LOCAL_H
#define DEFER_GOOCH_LOCAL_H
/* Singleton and derived class of State pattern. Class for deferred shading with Gooch illumination and
   point lights with local influence */


#include "DeferShader.h"


class DeferGoochLocal: public DeferShader {
    public:
        //Get a pointer to the DeferGoochLocal singleton (and construct if this hasn't been done yet)
        static DeferGoochLocal* getInstance(Geometry* geoLights);

        //Get a pointer to the DeferGoochLocal singleton only (no construction happens)
        static DeferGoochLocal* getPointer();

        void deactivate();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static DeferGoochLocal* singleton;
        //Gooch 'post-processing'
        Fbo* fboGooch;
        CGprogram fp_Tex;
        CGparameter cg_TexGooch;
        GLuint texGooch;


    private:
        //Constructor
        DeferGoochLocal(Geometry* geoLights, std::string gp_fileName, std::string fp_fileName, std::string stateInfo_);
};

#endif
