#ifndef DEFER_GOOCH_SPOT_H
#define DEFER_GOOCH_SPOT_H
/* Singleton and derived class of State pattern. Class for deferred shading with Gooch illumination and
   spot lights */


#include "DeferShader.h"


class DeferGoochSpot: public DeferShader {
    public:
        //Get a pointer to the DeferGoochSpot singleton (and construct if this hasn't been done yet)
        static DeferGoochSpot* getInstance(Geometry* geoLights);

        //Get a pointer to the DeferGoochSpot singleton only (no construction happens)
        static DeferGoochSpot* getPointer();

        void deactivate();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static DeferGoochSpot* singleton;
        //Gooch 'post-processing'
        Fbo* fboGooch;
        CGprogram fp_Tex;
        CGparameter cg_TexGooch;
        GLuint texGooch;


    private:
        //Constructor
        DeferGoochSpot(Geometry* geoLights, std::string gp_fileName, std::string fp_fileName, std::string stateInfo_);
};

#endif
