#ifndef DEFER_PHONG_GLOBAL_H
#define DEFER_PHONG_GLOBAL_H
/* Singleton and derived class of State pattern. Class for deferred shading with Phong illumination and
   point lights with global influence */


#include "DeferShader.h"


class DeferPhongGlobal: public DeferShader {
    public:
        //Get a pointer to the DeferPhongGlobal singleton (and construct if this hasn't been done yet)
        static DeferPhongGlobal* getInstance(Geometry* geoLights);

        //Get a pointer to the DeferPhongGlobal singleton only (no construction happens)
        static DeferPhongGlobal* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static DeferPhongGlobal* singleton;


    private:
        //Constructor
        DeferPhongGlobal(Geometry* geoLights, std::string gp_fileName, std::string fp_fileName, std::string stateInfo_): DeferShader(geoLights, gp_fileName, fp_fileName, stateInfo_) {}
};

#endif
