#ifndef DEFER_PHONG_LOCAL_H
#define DEFER_PHONG_LOCAL_H
/* Singleton and derived class of State pattern. Class for deferred shading with Phong illumination and
   point lights with local influence */


#include "DeferShader.h"


class DeferPhongLocal: public DeferShader {
    public:
        //Get a pointer to the DeferPhongLocal singleton (and construct if this hasn't been done yet)
        static DeferPhongLocal* getInstance(Geometry* geoLights);

        //Get a pointer to the DeferPhongLocal singleton only (no construction happens)
        static DeferPhongLocal* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static DeferPhongLocal* singleton;


    private:
        //Constructor
        DeferPhongLocal(Geometry* geoLights, std::string gp_fileName, std::string fp_fileName, std::string stateInfo_): DeferShader(geoLights, gp_fileName, fp_fileName, stateInfo_) {}
};

#endif
