#ifndef DEFER_PHONG_SPOT_H
#define DEFER_PHONG_SPOT_H
/* Singleton and derived class of State pattern. Class for deferred shading with Phong illumination and
   spot lights */


#include "DeferShader.h"


class DeferPhongSpot: public DeferShader {
    public:
        //Get a pointer to the DeferPhongSpot singleton (and construct if this hasn't been done yet)
        static DeferPhongSpot* getInstance(Geometry* geoLights);

        //Get a pointer to the DeferPhongSpot singleton only (no construction happens)
        static DeferPhongSpot* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static DeferPhongSpot* singleton;


    private:
        //Constructor
        DeferPhongSpot(Geometry* geoLights, std::string gp_fileName, std::string fp_fileName, std::string stateInfo_): DeferShader(geoLights, gp_fileName, fp_fileName, stateInfo_) {}
};

#endif
