#ifndef DEFER_SHADER_H
#define DEFER_SHADER_H
/* Base class for deferred shading */


#include "ShadingState.h"
#include "ShaderProgFactory.h"
class Geometry;
class Window;
class Fbo;
class FboManager;
class FullScreenQuad;


class DeferShader: public ShadingState {
    public:
        Window* window;
        ShaderProgFactory* shaderProgFactory;
        FullScreenQuad* fullScreenQuad;
        CGparameter cg_AlphaReject;  //Threshold for rejecting fragment with an lower alpha
        //Light data
        Geometry* geoLights;
        //Gbuffer
        FboManager* fboManager;
        Fbo* fbo;
        CGprogram vp_ObjToEye, fp_Gbuffer;
        GLuint texSceneColor, texScenePos, texSceneNormal;
        CGparameter cg_Material, cg_TexColor;
        //Ambient shading
        CGprogram vp_NoTransform, fp_Ambient;
        CGparameter cg_TexSceneColor_Ambient;  //Texture with scene color
        //Diffuse and specular shading
        CGprogram gp_LightVolume, fp_DeferShader;
        CGparameter cg_TexSceneColor, cg_TexScenePos, cg_TexSceneNormal;  //Textures with scene color, positions, and normal


    public:
        //Constructor
        DeferShader(Geometry* geoLights_, std::string gp_fileName, std::string fp_fileName, std::string stateInfo_);

        //Draw the scene with ambient shading (by multiplying ambient term with the values in the color buffer)
        void drawAmbientScene();

        //Activate deferred shading
        void activate();

        //Deactivate deferred shading
        void deactivate();

        //Enable color texture of geometry to be drawn
        void enableTexColor(GLuint texColor);

        //Disable color texture of geometry
        void disableTexColor();

        //Reject fragments with an alpha value lower than parameter alphaReject
        void enableAlphaReject(GLfloat alphaReject);

        //Stop rejecting fragments with a specific alpha value
        void disableAlphaReject();

        //Set the number of lights to be used
        void setNumLights(GLushort numLights_);
};

#endif
