#include <cstdlib>
#include "Eye.h"
#include "Geometry.h"
#include "Timer.h"
#include "objload.h"
#include "LightTornados.h"
#include "ShadingState.h"
#include "Demonstration.h"


//Constructor
Demonstration::Demonstration() {
    eye                        = Eye::getInstance(-215.0, 20.0, 0.0, 270.0);
    lightTornados              = LightTornados::getInstance(500, 5, 80.0, 15.0);
    lightTornados->activeState = lightTornados->activeState->enableTraditional();
    useTraditional             = true;  //Start with traditional shading
    //Setup the scene
    scene[0]          = setup3dModelUnitized(  0.0, 160.0,   0.0,   0.0,240.0, "3dModels/sky.obj",            "3dModels/panorama.png");
    scene[1]          = setup3dModelUnitized(  0.0,   0.0,   0.0,   0.0,  1.0, "3dModels/ground.obj",         "3dModels/generic_grass4_col.png");
    scene[1]->setScale(230.0, 75.0, 230.0);
    scene[2]          = setup3dModel(          79.0,  7.5,   0.0, 104.0,  5.3, "3dModels/platform2Rock.obj",  "3dModels/stone_cliff_tile_001_col.png");
    scene[3]          = setup3dModel(          79.0,  7.5,   0.0, 104.0,  5.3, "3dModels/platform2Grass.obj", "3dModels/grass.png");
    scene[4]          = setup3dModel(          48.0, 10.5,  61.5,   0.0,  7.1, "3dModels/platform3Rock.obj",  scene[2]->tex);
    scene[5]          = setup3dModel(          48.0, 10.5,  61.5,   0.0,  7.1, "3dModels/platform3Grass.obj", scene[3]->tex);
    scene[6]          = setup3dModelUnitized(  -6.0,  2.9,  78.0, 296.0, 15.0, "3dModels/platform.obj",       "3dModels/prop_rocks_rocksgrass_001_col.png");
    scene[7]          = setup3dModelUnitized(  17.0, -1.6, 110.5,   0.0,  9.5, "3dModels/flowers6.obj",       "3dModels/prop_plants_ground_002_col.png");
    scene[8]          = setup3dModelUnitized( -38.0, -0.8,  86.5, 292.0,  9.5, "3dModels/flowers4.obj",       scene[7]->tex);
    scene[8]->setRotationAxis(-0.04, 1.0, 0.04);
    scene[9]          = setup3dModelUnitized( -60.0,  1.0,  53.5,   0.0, 10.0, "3dModels/flowers.obj",        scene[7]->tex);
    scene[9]->setRotationAxis(0.0, 1.0, -0.82);
    scene[10]         = setup3dModelUnitized( -82.5,  2.6,  31.5,  24.0,  9.5, "3dModels/flowers6.obj",      scene[7]->tex);
    scene[11]         = setup3dModelUnitized( -60.0,  2.9,   2.5,   0.0,  9.5, "3dModels/flowers6.obj",       scene[7]->tex);
    scene[12]         = setup3dModelUnitized( 110.0, -0.1, -12.5,  84.0,  9.5, "3dModels/flowers.obj",        scene[7]->tex);
    scene[12]->setRotationAxis(0.0, 1.0, 0.02);
    scene[13]         = setup3dModelUnitized( -23.5,  1.5, -21.0, 229.0,  9.5, "3dModels/flowers6.obj",       scene[7]->tex);
    scene[14]         = setup3dModelUnitized(  47.5,  1.2, -22.0,   0.0,  9.5, "3dModels/flowers5.obj",       scene[7]->tex);
    scene[15]         = setup3dModelUnitized( -99.0,  1.9, -36.5,   0.0,  9.5, "3dModels/flowers2.obj",       scene[7]->tex);
    scene[16]         = setup3dModelUnitized( -37.0,  1.4, -91.0, 166.0,  9.5, "3dModels/flowers4.obj",       scene[7]->tex);
    scene[17]         = setup3dModelUnitized(  53.0,  2.1, -78.0,   0.0,  9.5, "3dModels/flowers6.obj",       scene[7]->tex);
    scene[17]->setRotationAxis(1.0, 1.0, 0.0);
    scene[18]         = setup3dModelUnitized( -51.5,  1.2, -87.5,   0.0,  9.5, "3dModels/flowers3.obj",       scene[7]->tex);
    scene[19]         = setup3dModelUnitized( -82.0,  2.1,  51.0,   0.0,  9.5, "3dModels/flowers5.obj",       scene[7]->tex);
    scene[20]         = setup3dModelUnitized(   0.5,  2.1,  24.0,  54.0,  9.5, "3dModels/flowers7.obj",       scene[7]->tex);
    scene[20]->setRotationAxis(0.06, 1.0, 0.0);
    sceneAlphaTest[0] = setup3dModelUnitized( -36.0,  2.1,  78.0, 236.0,  4.0, "3dModels/flowers8.obj",       scene[7]->tex);
    sceneAlphaTest[1] = setup3dModelUnitized(-104.5,  3.8, -21.0,   0.0,  2.4, "3dModels/flowers8.obj",       scene[7]->tex);
    sceneAlphaTest[2] = setup3dModelUnitized( -86.0,  4.1, -58.0,   0.0,  3.0, "3dModels/flowers9.obj",       scene[7]->tex);
    scene[21]         = setup3dModelUnitized( -80.0,  8.2,  24.0, 128.0,  6.5, "3dModels/fenceShort.obj",     "3dModels/prop_fence1_col.png");
    scene[22]         = setup3dModelUnitized( -90.0,  5.8,  -1.0, 101.0,  7.0, "3dModels/fenceLong.obj",      scene[21]->tex);
    scene[23]         = setup3dModelUnitized( -90.0,  6.1, -32.0,  80.0,  8.0, "3dModels/fenceLong.obj",      scene[21]->tex);
    scene[24]         = setup3dModelUnitized( -75.0,  7.2, -55.0,  40.0,  6.0, "3dModels/fenceShort.obj",     scene[21]->tex);
    scene[25]         = setup3dModel(           0.0,  5.6,  10.0,   0.0,  4.0, "3dModels/tree4Bark002.obj",   "3dModels/trees_bark_002_col.png");
    sceneAlphaTest[3] = setup3dModel(           0.0,  5.6,  10.0,   0.0,  4.0, "3dModels/tree4Leaves003.obj", "3dModels/trees_leaves_003_col.png");
    scene[26]         = setup3dModelUnitized( -55.0,  5.6, -15.0, 291.0,  4.0, "3dModels/ram.obj",            "3dModels/char_ram_col.png");
    scene[27]         = setup3dModelUnitized( -65.0,  3.7,  12.0, 302.0,  3.0, "3dModels/sheep.obj",          "3dModels/char_sheep_col.png");
    scene[28]         = setup3dModelUnitized( -75.0,  3.7,  -5.0, 275.0,  3.0, "3dModels/sheep2.obj",         scene[27]->tex);
    scene[29]         = setup3dModelUnitized( -75.0,  3.7, -25.0, 100.0,  3.0, "3dModels/sheep.obj",          scene[27]->tex);
    scene[30]         = setup3dModelUnitized( -65.0,  3.5, -40.0, 237.0,  3.0, "3dModels/sheep2.obj",         scene[27]->tex);
    scene[31]         = setup3dModelUnitized(  10.8, -0.4,  84.2, 229.0,  1.7, "3dModels/mushroom6.obj",      "3dModels/prop_mushrooms_001_col.png");
    scene[32]         = setup3dModelUnitized(  -6.1, -1.1,  97.7, 154.0,  1.7, "3dModels/mushroom3.obj",      scene[31]->tex);
    scene[33]         = setup3dModelUnitized(  -9.6,  0.2,  60.4,  43.0,  1.7, "3dModels/mushroom2.obj",      scene[31]->tex);
    scene[34]         = setup3dModelUnitized(  37.0,  0.4,  71.0,  53.0,  1.7, "3dModels/mushroom5.obj",      scene[31]->tex);
    scene[35]         = setup3dModelUnitized(  48.5,  0.3,  77.0, 264.0,  1.7, "3dModels/mushroom2.obj",      scene[31]->tex);
    scene[36]         = setup3dModelUnitized(  56.0,  0.4,  73.0, 285.0,  1.7, "3dModels/mushroom.obj",       scene[31]->tex);
    scene[37]         = setup3dModelUnitized(  58.0,  0.3,  54.0,   0.0,  1.7, "3dModels/mushroom4.obj",      scene[31]->tex);
    scene[38]         = setup3dModelUnitized(  90.5,  0.6,   4.5,   0.0,  1.7, "3dModels/mushroom3.obj",      scene[31]->tex);
    scene[39]         = setup3dModelUnitized(  74.0,  1.0, -11.5,   0.0,  1.7, "3dModels/mushroom4.obj",      scene[31]->tex);
    scene[40]         = setup3dModelUnitized(  71.0,  0.9,   6.0, 117.0,  1.7, "3dModels/mushroom6.obj",      scene[31]->tex);
    scene[41]         = setup3dModelUnitized(  51.5,  4.8, -42.0,  13.0,  1.7, "3dModels/mushroom.obj",       scene[31]->tex);
    scene[42]         = setup3dModelUnitized(  36.5,  1.6, -46.0,   0.0,  1.7, "3dModels/mushroom5.obj",      scene[31]->tex);
    scene[43]         = setup3dModelUnitized(  20.0, 19.4,  70.0, 110.0, 10.0, "3dModels/logSmall.obj",       "3dModels/trees_log_001_col.png");
    scene[43]->setRotationAxis(-0.06, 1.1, 0.3);
    scene[44]         = setup3dModelUnitized(  88.0,  2.0, -17.0,   0.0,  3.0, "3dModels/jumpObject.obj",     "3dModels/tree_stumps_col.png");
    scene[45]         = setup3dModelUnitized(  65.0, 23.3,  30.0,  62.0, 15.0, "3dModels/bridge.obj",         "3dModels/prop_bridge_col.png");
    sceneAlphaTest[4] = setup3dModelUnitized( -73.7, 11.4, -55.1, 217.0,  2.3, "3dModels/bird.obj",           "3dModels/char_bird_col.png");
    scene[46]         = setup3dModelUnitized(  -1.0,  6.1,   3.0, 194.0,  6.0, "3dModels/stone2.obj",         scene[2]->tex);
    scene[47]         = setup3dModelUnitized( -63.4,  7.6,  43.6,   0.0,  6.8, "3dModels/stone2.obj",         scene[2]->tex);
    scene[48]         = setup3dModelUnitized(  48.0,  0.7,  40.0, 154.0,  2.5, "3dModels/stone2.obj",         scene[2]->tex);
    scene[49]         = setup3dModelUnitized(  82.0,  0.6,  52.0,  43.0,  2.9, "3dModels/stone.obj",          scene[2]->tex);
    scene[50]         = setup3dModelUnitized(  36.5,  0.7,  -4.0,  53.0,  4.3, "3dModels/stone3.obj",         scene[2]->tex);
    scene[51]         = setup3dModelUnitized( -45.0,  3.1, -78.0, 264.0,  3.0, "3dModels/stone2.obj",         scene[2]->tex);
    scene[52]         = setup3dModelUnitized(-101.0,  3.1,  -9.5, 285.0,  2.8, "3dModels/stone.obj",          scene[2]->tex);
    scene[53]         = setup3dModelUnitized(  56.5,  1.9, -71.5,   0.0,  2.2, "3dModels/stone.obj",          scene[2]->tex);
    scene[54]         = setup3dModelUnitized( -67.2, 11.5,  33.6, 316.0,  7.9, "3dModels/rabbit.obj",         "3dModels/rabbit_skin_eye_col.png");
    scene[55]         = setup3dModel(         -20.0,  2.0, -50.0,   0.0,  6.0, "3dModels/treeBark002.obj",    scene[25]->tex);
    sceneAlphaTest[5] = setup3dModel(         -20.0,  2.0, -50.0,   0.0,  6.0, "3dModels/treeLeaves003.obj",  sceneAlphaTest[3]->tex);
    scene[56]         = setup3dModelUnitized(  45.8,  1.1, -45.0, 236.0,  6.2, "3dModels/logSmall.obj",       "3dModels/trees_log_001_col.png");
    sceneAlphaTest[6] = setup3dModelUnitized(  39.2,  5.0, -39.0, 117.0,  6.2, "3dModels/logBig.obj",         scene[56]->tex);
    sceneAlphaTest[6]->setRotationAxis(-0.2, 1.16, 0.04);
    scene[57]         = setup3dModelUnitized(   0.0,  3.8, -80.0,  10.0,  4.0, "3dModels/flyingsquirrel.obj", "3dModels/flyingsquirrel_skin_tail_col.png");
    scene[58]         = setup3dModelUnitized(  16.0,  5.1, -76.0,  48.0,  4.0, "3dModels/chinchilla.obj",     "3dModels/chinchilla_skin_eye_col.png");
    scene[59]         = setup3dModelUnitized( -10.0,  5.2, -70.0,   0.0,  4.0, "3dModels/squirrel.obj",       "3dModels/squirrel_fur_eye_col.png");
    scene[60]         = setup3dModelUnitized(  81.0, 29.1,   0.0, 152.0,  4.0, "3dModels/monkey.obj",         "3dModels/char_momo_col.png");
    sceneAlphaTest[7] = setup3dModelUnitized(   4.3, 23.0,  29.3, 276.0,  2.5, "3dModels/bird2.obj",          sceneAlphaTest[4]->tex);
    sceneAlphaTest[7]->setRotationAxis(0.12, 1.0, 0.0);
    scene[61]         = setup3dModelUnitized( -10.0,  0.5,  29.0,  56.0, 1.0, "3dModels/apricot.obj",         "3dModels/props_pickups_apricot_diff.png");
    scene[62]         = setup3dModelUnitized( -26.0,  1.3,  22.0,  47.0, 1.0, "3dModels/apricot.obj",         scene[61]->tex);
    scene[63]         = setup3dModelUnitized(  18.0,  0.7,  15.0, 226.0, 1.0, "3dModels/apricot.obj",         scene[61]->tex);
    scene[64]         = setup3dModelUnitized( -32.0,  1.5,  13.0,  19.0, 1.0, "3dModels/apricot.obj",         scene[61]->tex);
    scene[65]         = setup3dModelUnitized( -16.0,  1.0,   1.0, 217.0, 1.0, "3dModels/apricot.obj",         scene[61]->tex);
    scene[66]         = setup3dModelUnitized(   4.0,  0.6,  18.0, 136.0, 1.0, "3dModels/apricot.obj",         scene[61]->tex);
    scene[67]         = setup3dModelUnitized( -33.0,  1.1, -42.0, 220.0, 1.0, "3dModels/apricot.obj",         scene[61]->tex);
    scene[68]         = setup3dModelUnitized( -13.0,  0.5, -51.0, 359.0, 1.0, "3dModels/apricot.obj",         scene[61]->tex);
    scene[69]         = setup3dModelUnitized( -41.0,  1.2, -58.0, 287.0, 1.0, "3dModels/apricot.obj",         scene[61]->tex);
    scene[70]         = setup3dModelUnitized( -22.0,  0.7, -60.0, 146.0, 1.0, "3dModels/apricot.obj",         scene[61]->tex);
    scene[71]         = setup3dModelUnitized( -34.0,  0.9, -64.0,   6.0, 1.0, "3dModels/apricot.obj",         scene[61]->tex);
    scene[72]         = setup3dModelUnitized(   2.0,  0.7, -46.0, 328.0, 1.0, "3dModels/apricot.obj",         scene[61]->tex);
    scene[73]         = setup3dModelUnitized(  -8.0,  0.6, -36.0,  87.0, 1.0, "3dModels/apricot.obj",         scene[61]->tex);
    scene[74]         = setup3dModelUnitized(  21.0,  0.5,  73.0, 311.0, 4.0, "3dModels/rat.obj",             "3dModels/rat_col.png");
    scene[75]         = setup3dModelUnitized(  32.0,  0.0,  83.0, 194.0, 4.0, "3dModels/rat3.obj",            scene[74]->tex);
    scene[76]         = setup3dModelUnitized(  84.0,  1.1,  20.0, 136.0, 4.0, "3dModels/rat.obj",             scene[74]->tex);
    scene[77]         = setup3dModelUnitized(  57.0,  0.7,  17.0, 223.0, 4.0, "3dModels/rat2.obj",            scene[74]->tex);
    scene[78]         = setup3dModelUnitized(  66.0,  0.8,  61.0, 329.0, 4.0, "3dModels/rat3.obj",            scene[74]->tex);

    //Draw all 3D models of the scene
    sceneComplexity          = 79;
    sceneAlphaTestComplexity = 8;

    //Set time variables
    timer        = Timer::getInstance();
    tEye         = timer->time;
    tInterval    = timer->time;
    funcTimeStep = &Demonstration::intervalUserInteraction;  //Start with the animation which adds lights
}


//draw
void Demonstration::draw() {
    lightTornados->activate();
    for (GLushort i = 0; i < sceneComplexity; ++i) {
        lightTornados->activeState->enableTexColor(scene[i]->tex);
        scene[i]->draw();
    }
    //Draw scene which requires alpha testing
    lightTornados->activeState->enableAlphaReject(0.52);
    for (GLushort i = 0; i < sceneAlphaTestComplexity; ++i) {
        lightTornados->activeState->enableTexColor(sceneAlphaTest[i]->tex);
        sceneAlphaTest[i]->draw();
    }
    lightTornados->activeState->disableAlphaReject();
    lightTornados->deactivate();
}


//setup3dModel
Geometry* Demonstration::setup3dModel(GLfloat xPos, GLfloat yPos, GLfloat zPos, GLfloat yRotation, GLfloat scale, std::string fileName3dModel, std::string fileNameTex) {
    GLuint numVertices, numIndices;
    GLuint* indices;
    GLfloat* positions, * normals, * tangents, * binormals, * texCoords;
    //Load 3D model in geometry object
    LoadObjModel(fileName3dModel.c_str(), numVertices, numIndices, indices, positions, normals, tangents, binormals, texCoords);
    Geometry* geo3dModel = new Geometry(numVertices, numIndices, fileNameTex.c_str());
    geo3dModel->setPositions(XYZ, positions, GL_STATIC_DRAW);
    geo3dModel->setTexCoords(XY, texCoords, GL_STATIC_DRAW);
    geo3dModel->setNormals(normals, GL_STATIC_DRAW);
    geo3dModel->setIndices(GL_TRIANGLES, indices, GL_STATIC_DRAW);
    geo3dModel->setTransform(xPos, yPos, zPos, yRotation, scale);
    //Set texture parameters
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    //Delete allocated memory
    delete[] indices; delete[] positions; delete[] normals; delete[] tangents; delete[] binormals; delete[] texCoords;
    return geo3dModel;
}


//setup3dModel
Geometry* Demonstration::setup3dModel(GLfloat xPos, GLfloat yPos, GLfloat zPos, GLfloat yRotation, GLfloat scale, std::string fileName3dModel, GLuint tex) {
    GLuint numVertices, numIndices;
    GLuint* indices;
    GLfloat* positions, * normals, * tangents, * binormals, * texCoords;
    //Load 3D model in geometry object
    LoadObjModel(fileName3dModel.c_str(), numVertices, numIndices, indices, positions, normals, tangents, binormals, texCoords);
    Geometry* geo3dModel = new Geometry(numVertices, numIndices, tex);
    geo3dModel->setPositions(XYZ, positions, GL_STATIC_DRAW);
    geo3dModel->setTexCoords(XY, texCoords, GL_STATIC_DRAW);
    geo3dModel->setNormals(normals, GL_STATIC_DRAW);
    geo3dModel->setIndices(GL_TRIANGLES, indices, GL_STATIC_DRAW);
    geo3dModel->setTransform(xPos, yPos, zPos, yRotation, scale);
    //Delete allocated memory
    delete[] indices; delete[] positions; delete[] normals; delete[] tangents; delete[] binormals; delete[] texCoords;
    return geo3dModel;
}


//setup3dModel
Geometry* Demonstration::setup3dModelUnitized(GLfloat xPos, GLfloat yPos, GLfloat zPos, GLfloat yRotation, GLfloat scale, std::string fileName3dModel, std::string fileNameTex) {
    GLuint numVertices, numIndices;
    GLuint* indices;
    GLfloat* positions, * normals, * tangents, * binormals, * texCoords;
    //Load 3D model in geometry object
    LoadObjModelUnitized(fileName3dModel.c_str(), numVertices, numIndices, indices, positions, normals, tangents, binormals, texCoords);
    Geometry* geo3dModel = new Geometry(numVertices, numIndices, fileNameTex.c_str());
    geo3dModel->setPositions(XYZ, positions, GL_STATIC_DRAW);
    geo3dModel->setTexCoords(XY, texCoords, GL_STATIC_DRAW);
    geo3dModel->setNormals(normals, GL_STATIC_DRAW);
    geo3dModel->setIndices(GL_TRIANGLES, indices, GL_STATIC_DRAW);
    geo3dModel->setTransform(xPos, yPos, zPos, yRotation, scale);
    //Set texture parameters
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    //Delete allocated memory
    delete[] indices; delete[] positions; delete[] normals; delete[] tangents; delete[] binormals; delete[] texCoords;
    return geo3dModel;
}


//setup3dModelUnitized
Geometry* Demonstration::setup3dModelUnitized(GLfloat xPos, GLfloat yPos, GLfloat zPos, GLfloat yRotation, GLfloat scale, std::string fileName3dModel, GLuint tex) {
    GLuint numVertices, numIndices;
    GLuint* indices;
    GLfloat* positions, * normals, * tangents, * binormals, * texCoords;
    //Load 3D model in geometry object
    LoadObjModelUnitized(fileName3dModel.c_str(), numVertices, numIndices, indices, positions, normals, tangents, binormals, texCoords);
    Geometry* geo3dModel = new Geometry(numVertices, numIndices, tex);
    geo3dModel->setPositions(XYZ, positions, GL_STATIC_DRAW);
    geo3dModel->setTexCoords(XY, texCoords, GL_STATIC_DRAW);
    geo3dModel->setNormals(normals, GL_STATIC_DRAW);
    geo3dModel->setIndices(GL_TRIANGLES, indices, GL_STATIC_DRAW);
    geo3dModel->setTransform(xPos, yPos, zPos, yRotation, scale);
    //Delete allocated memory
    delete[] indices; delete[] positions; delete[] normals; delete[] tangents; delete[] binormals; delete[] texCoords;
    return geo3dModel;
}


//animationStep
void Demonstration::timeStep() {
    (this->*funcTimeStep)();
}


//enableUserInteraction
void Demonstration::enableUserInteraction() {
    funcTimeStep = &Demonstration::intervalUserInteraction;
    tInterval = timer->time;
}


//intervalUser
void Demonstration::intervalUserInteraction() {
    if (timer->time - tInterval > 30.0) {
        //Switch to animation
        eye->resetView(-215.0, 20.0, 0.0);  //Reset back to initial view
        funcTimeStep = &Demonstration::intervalAnimationAddLights;
        tInterval = timer->time;
        tEye = timer->time;
        tLights = timer->time;
    }
}


//intervalAnimationAddLights
void Demonstration::intervalAnimationAddLights() {
    GLfloat rotation = 10.0 * (timer->time - tEye);
    tEye = timer->time;
    eye->rotateLeft(rotation);

    if (timer->time - tLights > 0.3) {
        if (timer->fps > 55.0)
            lightTornados->addLights((int)(timer->fps * 0.06));  //Add lights
        if (timer->time - tInterval > 20.0)
            funcTimeStep = &Demonstration::intervalAnimationDeleteLights;  //Switch to animation which delete lights
        tLights = timer->time;
    }
}


//intervalAnimationDeleteLights
void Demonstration::intervalAnimationDeleteLights() {
    GLfloat rotation = 10.0 * (timer->time - tEye);
    tEye = timer->time;
    eye->rotateLeft(rotation);

    if (timer->time - tLights > 0.3) {
        //Delete lights
        double remainingTime = 30.0 - (timer->time - tInterval);
        lightTornados->deleteLights((lightTornados->numLights - 5) / remainingTime);
        if (timer->time - tInterval > 30.0) {
            //Switch to animation which adds lights
            funcTimeStep = &Demonstration::intervalAnimationAddLights;
            if (useTraditional) {
                //Switch to deferred shading
                lightTornados->activeState = lightTornados->activeState->enableDeferred();
                useTraditional = false;
            }
            else {
                //Switch to traditional shading
                lightTornados->activeState = lightTornados->activeState->enableTraditional();
                useTraditional = true;
                //Switch to a random illumination model, type of lights, and scene
                int rIllum = rand() % 4;  //Illumination model
                int rType  = rand() % 3;  //Type of lights (spot, local point, or global point)
                switch(rIllum) {
                    case 0: lightTornados->activeState = lightTornados->activeState->enablePhong(); break;
                    case 1: lightTornados->activeState = lightTornados->activeState->enableBlinnPhong(); break;
                    case 2: lightTornados->activeState = lightTornados->activeState->enableCookTorrance(); break;
                    case 3: lightTornados->activeState = lightTornados->activeState->enableGooch(); break;
                }
                switch(rType) {
                    case 0: lightTornados->activeState = lightTornados->activeState->enableLocal(); break;
                    case 1: lightTornados->activeState = lightTornados->activeState->enableGlobal(); break;
                    case 2: lightTornados->activeState = lightTornados->activeState->enableSpot(); break;
                }
            }
            tInterval = timer->time;
        }
        tLights = timer->time;
    }
}


//getSwitchTime
int Demonstration::getSwitchTime() {
    return (int)(30.5 - timer->time + tInterval);  //Perform rounding
}


//addObject
void Demonstration::addObject() {
    if (sceneComplexity < 79)
        ++sceneComplexity;
    else if (sceneAlphaTestComplexity < 8)
        ++sceneAlphaTestComplexity;
}


//deleteObject
void Demonstration::deleteObject() {
    if (sceneAlphaTestComplexity > 0)
        --sceneAlphaTestComplexity;
    else if (sceneComplexity > 2)
        --sceneComplexity;
}
