#ifndef DEMONSTRATION_H
#define DEMONSTRATION_H
/* Class for demonstrating the capabilities of the application */


#include <string>
class Eye;
class Geometry;
class LightTornados;


class Demonstration {
    public:
        Eye* eye;
        //3D models making up the scene
        Geometry* scene[79];
        Geometry* sceneAlphaTest[8];
        GLushort sceneComplexity, sceneAlphaTestComplexity;
        LightTornados* lightTornados;
        //Time variables
        Timer* timer;
        double tInterval, tLights, tEye;        //Time for interval functions, for adding/deleting lights, and for rotating the eye
        void (Demonstration::*funcTimeStep)();  //Function pointer to animation or user interaction
        bool useTraditional;                    //Is traditional shading currently used or not


    public:
        //Constructor. Load all 3D models and puts them in array 'scene'
        Demonstration();

        //Load and place 3D model in scene
        Geometry* setup3dModel(GLfloat xPos, GLfloat yPos, GLfloat zPos, GLfloat yRotation, GLfloat scale, std::string fileName3dModel, std::string fileNameTex);
        Geometry* setup3dModel(GLfloat xPos, GLfloat yPos, GLfloat zPos, GLfloat yRotation, GLfloat scale, std::string fileName3dModel, GLuint tex);
        Geometry* setup3dModelUnitized(GLfloat xPos, GLfloat yPos, GLfloat zPos, GLfloat yRotation, GLfloat scale, std::string fileName3dModel, std::string fileNameTex);
        Geometry* setup3dModelUnitized(GLfloat xPos, GLfloat yPos, GLfloat zPos, GLfloat yRotation, GLfloat scale, std::string fileName3dModel, GLuint tex);

        //Draw demonstration
        void draw();

        //Perform a step in the animation
        void timeStep();

        //Switch to user interaction
        void enableUserInteraction();

        //Function that handles user interaction. After several seconds without user input, the animation starts again
        void intervalUserInteraction();

        //Functions that show an animation.
        void intervalAnimationAddLights();
        void intervalAnimationDeleteLights();

        //Get remaining time before switching with animation
        int getSwitchTime();

        //Add/delete object from scene
        void addObject();
        void deleteObject();
};

#endif
