#include "Eye.h"
#include <GL/glut.h>


#define LEFT_DOWN   1
#define MIDDLE_DOWN 2
#define RIGHT_DOWN  3
Eye* Eye::singleton = NULL;


//Constructor
Eye::Eye(GLfloat distance_, GLfloat elevation_, GLfloat azimuth_, GLfloat maxDistance_) {
    mouseButton = 0;
    distance    = distance_;
    elevation   = elevation_;
    azimuth     = azimuth_;
    maxDistance = maxDistance_;
    setView();
}


//getInstance
Eye* Eye::getInstance(GLfloat distance, GLfloat elevation, GLfloat azimuth, GLfloat maxDistance) {
    if (singleton == NULL)
        singleton = new Eye(distance, elevation, azimuth, maxDistance);
    return singleton;
}


//getPointer
Eye* Eye::getPointer() {
    return singleton;
}


//setView
void Eye::setView() {
    if (distance > maxDistance)
        distance = maxDistance;
    else if (distance < -maxDistance)
        distance = -maxDistance;
    if (elevation < 5.0)
        elevation = 5.0;
    else if (elevation > 165.0)
        elevation = 165.0;

    glLoadIdentity();
    glTranslatef(0.0, 0.0, distance);         //Zoom
    glRotatef(elevation, 1.0, 0.0, 0.0);      //Rotation about the x-axis
    glRotatef(azimuth, 0.0, 1.0, 0.0);        //Rotation about the y-axis
    glutPostRedisplay();
}


//resetView
void Eye::resetView(GLfloat distance_, GLfloat elevation_, GLfloat azimuth_) {
    distance    = distance_;
    elevation   = elevation_;
    azimuth     = azimuth_;
    setView();
}


//rotateLeft
void Eye::rotateLeft(GLfloat rotation) {
    azimuth += rotation;
    setView();
}


//mouseButtonPressed
void Eye::mouseButtonPressed(int button, int state, int x, int y) {
    if (state == GLUT_DOWN) {
        //Store mouse positions
        xprev = x;
        yprev = y;

        if (button == GLUT_LEFT_BUTTON)
            mouseButton = LEFT_DOWN;
        else if (button == GLUT_MIDDLE_BUTTON)
            mouseButton = MIDDLE_DOWN;
        else if (button == GLUT_RIGHT_BUTTON)
            mouseButton = RIGHT_DOWN;
        else
            mouseButton = 0;
    }
}


//mouseMotion
void Eye::mouseMotion(int x, int y) {
    if (mouseButton == LEFT_DOWN)
        azimuth += (x - xprev);
    else if (mouseButton == MIDDLE_DOWN)
        distance += (y - yprev) * 0.2;
    else if (mouseButton == RIGHT_DOWN) {
        elevation += (y - yprev) * 0.5;
    }
    setView();

    xprev = x;
    yprev = y;
    glutPostRedisplay();
}
