#ifndef EYE_H
#define EYE_H
/* Singleton. Azimuth-elevation view controls. Eye remains at origin, it is the scene that is translated/rotated */


#include <GL/glew.h>


class Eye {
    public:
        GLubyte mouseButton;
        int xprev, yprev;                      //Previous coordinates of mouse cursor
        GLfloat distance, azimuth, elevation;  //View parameters
        GLfloat maxDistance;                   //Limit maximum distance to center


    public:
        //Setup the position and view direction of the eye (and construct if this hasn't been done yet)
        static Eye* getInstance(GLfloat distance, GLfloat elevation, GLfloat azimuth, GLfloat maxDistance_);

        //Get a pointer to the Eye singleton
        static Eye* getPointer();

        //Translate/rotate scene according to input
        void setView();

        //Reset view with the specified parameters
        void resetView(GLfloat distance_, GLfloat elevation_, GLfloat azimuth_);

        //Rotate the scene left with a given amount
        void rotateLeft(GLfloat rotation);

        //Determine which mouse button has been pressed down
        void mouseButtonPressed(int button, int state, int x, int y);

        //Determine new view according to mouse motion
        void mouseMotion(int x, int y);


    private:
        static Eye* singleton;


    private:
        //Constructor
        Eye(GLfloat eyeDist_, GLfloat eyeElev_, GLfloat eyeAzim_, GLfloat maxDistance_);
};

#endif
