#ifndef LIGHT_TORNADOS_H
#define LIGHT_TORNADOS_H
/* Class for initializing the light sources, performing a tornado animation on the ligth sources,
   and drawing the light sources */


#include "ShaderProgFactory.h"
class Geometry;
class Timer;
class TransformFeedback;
class ShadingState;
class ShadingStateFactory;


class LightTornados {
    public:
        ShaderProgFactory* shaderProgFactory;
        ShadingStateFactory* shadingStateFactory;
        ShadingState* activeState;
        //Light data
        GLushort maxLights;  //Maximum number of lights (lower -> faster)
        GLushort numLights;
        Geometry* geoLights;
        GLuint texLightSource;
        GLfloat outerRadius;
        //Animation tornados
        TransformFeedback* transformFeedback;
        Timer* timer;
        double tPrev;
        GLfloat outerAngle, addAngle;  //Angle of rotation, of the tornados themselves. And the amount added to this angle
        CGprogram vp_Tornado;
        CGparameter cg_TornadoPos, cg_OuterAngle, cg_InnerAngle, cg_AddAngle;


    public:
        //Get a pointer to the LightTornados singleton (and construct if this hasn't been done yet)
        static LightTornados* getInstance(GLushort maxLights_, GLushort numLights_, GLfloat outerRadius_, GLfloat innerRadius);

        //Get a pointer to the LightTornados singleton only (no construction happens)
        static LightTornados* getPointer();

        //Perform fountain animation step and then activate shading
        void activate();

        //Deactivate shading
        void deactivate();

        //Draw the light sources as point sprites
        void drawLightSources();

        //Tornado animation on positions of light sources
        void tornadoAnimation();

        //Add/delete lights
        void addLights(GLushort numAdded);
        void deleteLights(GLushort numDeleted);


    private:
        static LightTornados* singleton;


    private:
        //Constructor
        LightTornados(GLushort maxLights_, GLushort numLights_, GLfloat outerRadius, GLfloat innerRadius);
};

#endif