#include <cstdlib>
#include <iostream>
#include <GL/glew.h>
#include "ShaderProgFactory.h"


ShaderProgFactory* ShaderProgFactory::singleton;
CGcontext cg_Context;
CGprofile vertexProfile, geometryProfile, fragmentProfile;


//errorCallback
void errorCallback() {
    CGerror lastError = cgGetError();
    if (lastError) {
        std::cerr << cgGetErrorString(lastError) << "\n";
        std::cerr << cgGetLastListing(cg_Context) << "\n";
        exit(-1);
    }
}


//Constructor
ShaderProgFactory::ShaderProgFactory() {
    //Create a context for the shader program(s)
    cg_Context = cgCreateContext();
    if (cg_Context == 0) {
        std::cerr << "Failed to create Cg context\n";
        exit(-1);
    }
    //Specify error-callback function
    cgSetErrorCallback(errorCallback);
    //Set profiles. Use latest profiles
    vertexProfile   = cgGLGetLatestProfile(CG_GL_VERTEX);
    geometryProfile = cgGLGetLatestProfile(CG_GL_GEOMETRY);
    fragmentProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
    if (vertexProfile == CG_PROFILE_UNKNOWN || geometryProfile == CG_PROFILE_UNKNOWN || fragmentProfile == CG_PROFILE_UNKNOWN) {
        std::cerr << "Invalid profile type\n";
        exit(-1);
    }
    //Use optimal settings (dependent on compiler, GPU and drivers)
    cgGLSetOptimalOptions(vertexProfile);
    cgGLSetOptimalOptions(geometryProfile);
    cgGLSetOptimalOptions(fragmentProfile);
}


//getInstance
ShaderProgFactory* ShaderProgFactory::getInstance() {
    if (singleton == NULL)
        singleton = new ShaderProgFactory();
    return singleton;
}


//loadShader
CGprogram ShaderProgFactory::loadProgram(std::string fileName, std::string mainFunction, GLubyte programType) {
    switch (programType) {
        case VERTEX_PROGRAM:
            shaderProgram = cgCreateProgramFromFile(cg_Context, CG_SOURCE, fileName.c_str(), vertexProfile, mainFunction.c_str(), 0);
        break;
        case GEOMETRY_PROGRAM:
            shaderProgram = cgCreateProgramFromFile(cg_Context, CG_SOURCE, fileName.c_str(), geometryProfile, mainFunction.c_str(), 0);
        break;
        case FRAGMENT_PROGRAM:
            shaderProgram = cgCreateProgramFromFile(cg_Context, CG_SOURCE, fileName.c_str(), fragmentProfile, mainFunction.c_str(), 0);
        break;
    }
    cgGLLoadProgram(shaderProgram);
    return shaderProgram;
}


//createShader
CGprogram ShaderProgFactory::createProgram(std::string fileName, std::string mainFunction, GLubyte programType) {
    cgSetAutoCompile(cg_Context, CG_COMPILE_MANUAL);
    switch (programType) {
        case VERTEX_PROGRAM:
            shaderProgram = cgCreateProgramFromFile(cg_Context, CG_SOURCE, fileName.c_str(), vertexProfile, mainFunction.c_str(), 0);
        break;
        case GEOMETRY_PROGRAM:
            shaderProgram = cgCreateProgramFromFile(cg_Context, CG_SOURCE, fileName.c_str(), geometryProfile, mainFunction.c_str(), 0);
        break;
        case FRAGMENT_PROGRAM:
            shaderProgram = cgCreateProgramFromFile(cg_Context, CG_SOURCE, fileName.c_str(), fragmentProfile, mainFunction.c_str(), 0);
        break;
    }
    cgSetAutoCompile(cg_Context, CG_COMPILE_IMMEDIATE);
    return shaderProgram;
}
