#ifndef SHADING_STATE_H
#define SHADING_STATE_H
/* Base class of State pattern. Makes sure that we can switch to correct states from the current state */


#include <string>
#include <GL/glew.h>


class ShadingState {
    public:
        std::string stateInfo;
        GLushort numLights;


    public:
        virtual ShadingState* enableTraditional() = 0;

        virtual ShadingState* enableDeferred() = 0;

        virtual ShadingState* enableLocal() = 0;

        virtual ShadingState* enableSpot() = 0;

        virtual ShadingState* enableGlobal() = 0;

        virtual ShadingState* enablePhong() = 0;

        virtual ShadingState* enableBlinnPhong() = 0;

        virtual ShadingState* enableCookTorrance() = 0;

        virtual ShadingState* enableGooch() = 0;

        virtual void activate() = 0;

        virtual void deactivate() = 0;

        //Enable color texture of geometry to be drawn
        virtual void enableTexColor(GLuint texColor) = 0;

        //Disable color texture of geometry
        virtual void disableTexColor() = 0;

        //Reject fragments with an alpha value lower than parameter alphaReject
        virtual void enableAlphaReject(GLfloat alphaReject) = 0;

        //Stop rejecting fragments with a specific alpha value
        virtual void disableAlphaReject() = 0;

        //Set the number of lights to be used
        virtual void setNumLights(GLushort numLights_) = 0;
};

#endif
