#include <cstdlib>
#ifndef _WIN32
#    include <sys/time.h>
#endif
#include "Timer.h"


Timer* Timer::singleton = NULL;


#ifdef _WIN32
// ------------------------ Windows implementation ------------------------
//Constructor
Timer::Timer() {
    LARGE_INTEGER freq;
    QueryPerformanceFrequency(&freq);  //Get frequency of hi-resolution timer
    s         = 1.0 / freq.QuadPart;   //Store time interval
    fps       = 0.0;
    numFrames = 0;
    tPrev     = 0.0;
    QueryPerformanceCounter(&countStart);  //Start counting
}


//getInstance
Timer* Timer::getInstance() {
    if (singleton == NULL)
        singleton = new Timer();
    return singleton;
}


//getTime
void Timer::updateTime() {
    LARGE_INTEGER countCurr;
    QueryPerformanceCounter(&countCurr);
    countCurr.QuadPart = countCurr.QuadPart - countStart.QuadPart;
    time = countCurr.QuadPart * s;
}


//computeFps
void Timer::computeFps() {
    ++numFrames;
    LARGE_INTEGER countCurr;
    QueryPerformanceCounter(&countCurr);
    countCurr.QuadPart = countCurr.QuadPart - countStart.QuadPart;
    time = countCurr.QuadPart * s;
    //Compute fps every 0.29 sec
    if (time - tPrev > 0.29) {
        fps = numFrames / (time - tPrev);
        numFrames = 0;
        tPrev = time;
    }
}

#else
// ------------------------- Linux implementation -------------------------
//Constructor
Timer::Timer() {
    fps       = 0.0;
    numFrames = 0;
    timeval timCurr;
    gettimeofday(&timCurr, NULL);
    tStart = timCurr.tv_sec + (0.000001*timCurr.tv_usec);  //Starting time
    tPrev  = tStart;
}


//getInstance
Timer* Timer::getInstance() {
    if (singleton == NULL)
        singleton = new Timer();
    return singleton;
}


//updateTime
void Timer::updateTime() {
    timeval timCurr;
    gettimeofday(&timCurr, NULL);
    double t = timCurr.tv_sec + (0.000001*timCurr.tv_usec);
    time = t - tStart;
}


//computeFps
void Timer::computeFps() {
    ++numFrames;
    timeval timCurr;
    gettimeofday(&timCurr, NULL);
    double t = timCurr.tv_sec + (0.000001*timCurr.tv_usec);
	time = t - tStart;
    //Compute fps every 0.29 sec
    if (t - tPrev > 0.29) {
        fps = numFrames / (t - tPrev);
        numFrames = 0;
        tPrev = t;
    }
}

#endif
