#ifndef TRAD_BLINN_PHONG_GLOBAL_H
#define TRAD_BLINN_PHONG_GLOBAL_H
/* Singleton and derived class of State pattern. Class for traditional shading with Blinn-Phong illumination and
   point lights with global influence */


#include "TradShaderPoint.h"


class TradBlinnPhongGlobal: public TradShaderPoint {
    public:
        //Get a pointer to the TradBlinnPhongGlobal singleton (and construct if this hasn't been done yet)
        static TradBlinnPhongGlobal* getInstance(GLushort maxLights, Geometry* geoLights);

        //Get a pointer to the TradBlinnPhongGlobal singleton only (no construction happens)
        static TradBlinnPhongGlobal* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static TradBlinnPhongGlobal* singleton;


    private:
        //Constructor
        TradBlinnPhongGlobal(GLushort maxLights, Geometry* geoLights, std::string fileName, std::string stateInfo_): TradShaderPoint(maxLights, geoLights, fileName, stateInfo_) {}
};

#endif
