#ifndef TRAD_BLINN_PHONG_LOCAL_H
#define TRAD_BLINN_PHONG_LOCAL_H
/* Singleton and derived class of State pattern. Class for traditional shading with Blinn-Phong illumination and
   point lights with local influence */


#include "TradShaderPoint.h"


class TradBlinnPhongLocal: public TradShaderPoint {
    public:
        //Get a pointer to the TradBlinnPhongLocal singleton (and construct if this hasn't been done yet)
        static TradBlinnPhongLocal* getInstance(GLushort maxLights, Geometry* geoLights);

        //Get a pointer to the TradBlinnPhongLocal singleton only (no construction happens)
        static TradBlinnPhongLocal* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static TradBlinnPhongLocal* singleton;


    private:
        //Constructor
        TradBlinnPhongLocal(GLushort maxLights, Geometry* geoLights, std::string fileName, std::string stateInfo_): TradShaderPoint(maxLights, geoLights, fileName, stateInfo_) {}
};

#endif
