#ifndef TRAD_BLINN_PHONG_SPOT_H
#define TRAD_BLINN_PHONG_SPOT_H
/* Singleton and derived class of State pattern. Class for traditional shading with Blinn-Phong illumination and
   spot lights */


#include "TradShaderSpot.h"


class TradBlinnPhongSpot: public TradShaderSpot {
    public:
        //Get a pointer to the TradBlinnPhongSpot singleton (and construct it if this hasn't been done yet)
        static TradBlinnPhongSpot* getInstance(GLushort maxLights, Geometry* geoLights);

        //Get a pointer to the TradBlinnPhongSpot singleton only (no construction happens)
        static TradBlinnPhongSpot* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static TradBlinnPhongSpot* singleton;


    private:
        //Constructor
        TradBlinnPhongSpot(GLushort maxLights, Geometry* geoLights, std::string fileName, std::string stateInfo_): TradShaderSpot(maxLights, geoLights, fileName, stateInfo_) {}
};

#endif
