#ifndef TRAD_COOK_TORRANCE_GLOBAL_H
#define TRAD_COOK_TORRANCE_GLOBAL_H
/* Singleton and derived class of State pattern. Class for traditional shading with Cook-Torrance illumination and
   point lights with global influence */


#include "TradShaderPoint.h"


class TradCookTorranceGlobal: public TradShaderPoint {
    public:
        //Get a pointer to the TradCookTorranceGlobal singleton (and construct if this hasn't been done yet)
        static TradCookTorranceGlobal* getInstance(GLushort maxLights, Geometry* geoLights);

        //Get a pointer to the TradCookTorranceGlobal singleton only (no construction happens)
        static TradCookTorranceGlobal* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static TradCookTorranceGlobal* singleton;


    private:
        //Constructor
        TradCookTorranceGlobal(GLushort maxLights, Geometry* geoLights, std::string fileName, std::string stateInfo_): TradShaderPoint(maxLights, geoLights, fileName, stateInfo_) {}
};

#endif
