#ifndef TRAD_COOK_TORRANCE_LOCAL_H
#define TRAD_COOK_TORRANCE_LOCAL_H
/* Singleton and derived class of State pattern. Class for traditional shading with Cook-Torrance illumination and
   point lights with local influence */


#include "TradShaderPoint.h"


class TradCookTorranceLocal: public TradShaderPoint {
    public:
        //Get a pointer to the TradCookTorranceLocal singleton (and construct if this hasn't been done yet)
        static TradCookTorranceLocal* getInstance(GLushort maxLights, Geometry* geoLights);

        //Get a pointer to the TradCookTorranceLocal singleton only (no construction happens)
        static TradCookTorranceLocal* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static TradCookTorranceLocal* singleton;


    private:
        //Constructor
        TradCookTorranceLocal(GLushort maxLights, Geometry* geoLights, std::string fileName, std::string stateInfo_): TradShaderPoint(maxLights, geoLights, fileName, stateInfo_) {}
};

#endif
