#ifndef TRAD_COOK_TORRANCE_SPOT_H
#define TRAD_COOK_TORRANCE_SPOT_H
/* Singleton and derived class of State pattern. Class for traditional shading with Cook-Torrance illumination and
   spot lights */


#include "TradShaderSpot.h"


class TradCookTorranceSpot: public TradShaderSpot {
    public:
        //Get a pointer to the TradCookTorranceSpot singleton (and construct it if this hasn't been done yet)
        static TradCookTorranceSpot* getInstance(GLushort maxLights, Geometry* geoLights);

        //Get a pointer to the TradCookTorranceSpot singleton only (no construction happens)
        static TradCookTorranceSpot* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static TradCookTorranceSpot* singleton;


    private:
        //Constructor
        TradCookTorranceSpot(GLushort maxLights, Geometry* geoLights, std::string fileName, std::string stateInfo_): TradShaderSpot(maxLights, geoLights, fileName, stateInfo_) {}
};

#endif
