#ifndef TRAD_GOOCH_GLOBAL_H
#define TRAD_GOOCH_GLOBAL_H
/* Singleton and derived class of State pattern. Class for traditional shading with Gooch illumination and
   point lights with global influence */


#include "TradShaderPoint.h"


class TradGoochGlobal: public TradShaderPoint {
    public:
        //Get a pointer to the TradGoochGlobal singleton (and construct if this hasn't been done yet)
        static TradGoochGlobal* getInstance(GLushort maxLights, Geometry* geoLights);

        //Get a pointer to the TradGoochGlobal singleton only (no construction happens)
        static TradGoochGlobal* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static TradGoochGlobal* singleton;


    private:
        //Constructor
        TradGoochGlobal(GLushort maxLights, Geometry* geoLights, std::string fileName, std::string stateInfo_): TradShaderPoint(maxLights, geoLights, fileName, stateInfo_) {}
};

#endif
