#ifndef TRAD_GOOCH_LOCAL_H
#define TRAD_GOOCH_LOCAL_H
/* Singleton and derived class of State pattern. Class for traditional shading with Gooch illumination and
   point lights with local influence */


#include "TradShaderPoint.h"


class TradGoochLocal: public TradShaderPoint {
    public:
        //Get a pointer to the TradGoochLocal singleton (and construct if this hasn't been done yet)
        static TradGoochLocal* getInstance(GLushort maxLights, Geometry* geoLights);

        //Get a pointer to the TradGoochLocal singleton only (no construction happens)
        static TradGoochLocal* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static TradGoochLocal* singleton;


    private:
        //Constructor
        TradGoochLocal(GLushort maxLights, Geometry* geoLights, std::string fileName, std::string stateInfo_): TradShaderPoint(maxLights, geoLights, fileName, stateInfo_) {}
};

#endif
