#ifndef TRAD_GOOCH_SPOT_H
#define TRAD_GOOCH_SPOT_H
/* Singleton and derived class of State pattern. Class for traditional shading with Gooch illumination and
   spot lights */


#include "TradShaderSpot.h"


class TradGoochSpot: public TradShaderSpot {
    public:
        //Get a pointer to the TradGoochSpot singleton (and construct it if this hasn't been done yet)
        static TradGoochSpot* getInstance(GLushort maxLights, Geometry* geoLights);

        //Get a pointer to the TradGoochSpot singleton only (no construction happens)
        static TradGoochSpot* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static TradGoochSpot* singleton;


    private:
        //Constructor
        TradGoochSpot(GLushort maxLights, Geometry* geoLights, std::string fileName, std::string stateInfo_): TradShaderSpot(maxLights, geoLights, fileName, stateInfo_) {}
};

#endif
