#ifndef TRAD_PHONG_GLOBAL_H
#define TRAD_PHONG_GLOBAL_H
/* Singleton and derived class of State pattern. Class for traditional shading with Phong illumination and
   point lights with global influence */


#include "TradShaderPoint.h"


class TradPhongGlobal: public TradShaderPoint {
    public:
        //Get a pointer to the TradPhongGlobal singleton (and construct if this hasn't been done yet)
        static TradPhongGlobal* getInstance(GLushort maxLights, Geometry* geoLights);

        //Get a pointer to the TradPhongGlobal singleton only (no construction happens)
        static TradPhongGlobal* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static TradPhongGlobal* singleton;


    private:
        //Constructor
        TradPhongGlobal(GLushort maxLights, Geometry* geoLights, std::string fileName, std::string stateInfo_): TradShaderPoint(maxLights, geoLights, fileName, stateInfo_) {}
};

#endif
