#ifndef TRAD_PHONG_LOCAL_H
#define TRAD_PHONG_LOCAL_H
/* Singleton and derived class of State pattern. Class for traditional shading with Phong illumination and
   point lights with local influence */


#include "TradShaderPoint.h"


class TradPhongLocal: public TradShaderPoint {
    public:
        //Get a pointer to the TradPhongLocal singleton (and construct if this hasn't been done yet)
        static TradPhongLocal* getInstance(GLushort maxLights, Geometry* geoLights);

        //Get a pointer to the TradPhongLocal singleton only (no construction happens)
        static TradPhongLocal* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static TradPhongLocal* singleton;


    private:
        //Constructor
        TradPhongLocal(GLushort maxLights, Geometry* geoLights, std::string fileName, std::string stateInfo_): TradShaderPoint(maxLights, geoLights, fileName, stateInfo_) {}
};

#endif
