#ifndef TRAD_PHONG_SPOT_H
#define TRAD_PHONG_SPOT_H
/* Singleton and derived class of State pattern. Class for traditional shading with Phong illumination and
   spot lights */


#include "TradShaderSpot.h"


class TradPhongSpot: public TradShaderSpot {
    public:
        //Get a pointer to the TradPhongSpot singleton (and construct it if this hasn't been done yet)
        static TradPhongSpot* getInstance(GLushort maxLights, Geometry* geoLights);

        //Get a pointer to the TradPhongSpot singleton only (no construction happens)
        static TradPhongSpot* getPointer();

        //ShadingState functions for switching between states
        ShadingState* enableTraditional();
        ShadingState* enableDeferred();
        ShadingState* enableLocal();
        ShadingState* enableSpot();
        ShadingState* enableGlobal();
        ShadingState* enablePhong();
        ShadingState* enableBlinnPhong();
        ShadingState* enableCookTorrance();
        ShadingState* enableGooch();


    private:
        static TradPhongSpot* singleton;


    private:
        //Constructor
        TradPhongSpot(GLushort maxLights, Geometry* geoLights, std::string fileName, std::string stateInfo_): TradShaderSpot(maxLights, geoLights, fileName, stateInfo_) {}
};

#endif
