#include "Geometry.h"
#include "TransformFeedback.h"
#include "TradShaderPoint.h"


//Constructor
TradShaderPoint::TradShaderPoint(GLushort maxLights, Geometry* geoLights_, std::string fileName, std::string stateInfo_) {
    stateInfo = stateInfo_;
    geoLights = geoLights_;
    //Setup traditional shading with spot lights
    glGenTextures(1, &tboLightPos);
    shaderProgFactory = ShaderProgFactory::getInstance();
    vp_ObjToEye       = shaderProgFactory->loadProgram("Source/ShaderPrograms/vp_ObjToEye.cg", "main", VERTEX_PROGRAM);
    vp_LightsObjToEye = shaderProgFactory->loadProgram("Source/ShaderPrograms/TraditionalShading/vp_LightsObjToEye.cg", "main", VERTEX_PROGRAM);
    fp_TradShader     = shaderProgFactory->loadProgram(fileName, "main", FRAGMENT_PROGRAM);
    cg_Material       = cgGetNamedParameter(fp_TradShader, "material");
    cg_NumLights      = cgGetNamedParameter(fp_TradShader, "numLights");
    cg_TboLightPos    = cgGetNamedParameter(fp_TradShader, "tboLightPos");
    cg_TexColor       = cgGetNamedParameter(fp_TradShader, "texColor");
    cgGLSetParameter4f(cg_Material, 0.5, 1.0, 0.7, 16.0);  //Consists of ka, kd, ks, shininess
    cgGLSetTextureParameter(cg_TboLightPos, tboLightPos);
    //Setup transform-feedback
    transformFeedback = new TransformFeedback(maxLights, 1, GL_STREAM_DRAW);
    transformFeedback->enablePositions(4);  //Lights' positions
}


//lightsObjToEye
void TradShaderPoint::lightsObjToEye() {
    transformFeedback->bind(GL_POINTS);  //Render points to output buffers

    //Transform object-space lights to eye-space and stream result to TBO
    cgGLBindProgram(vp_LightsObjToEye);
    cgGLEnableProfile(vertexProfile);

    geoLights->draw(0, numLights);

    //Deactivate
    cgGLDisableProfile(vertexProfile);
    transformFeedback->unbind();

    //Attach eye-space positions and spot directions to TBOs
    glBindTexture(GL_TEXTURE_BUFFER_EXT, tboLightPos);
    glTexBufferEXT(GL_TEXTURE_BUFFER_EXT, GL_RGBA32F_ARB, *transformFeedback->positionBuffer);
}


//activate
void TradShaderPoint::activate() {
    lightsObjToEye();  //Object-space positions and directions of lights to eye-space
    //Activate traditional shading
    cgGLEnableTextureParameter(cg_TboLightPos);
    cgGLBindProgram(vp_ObjToEye); cgGLBindProgram(fp_TradShader);
    cgGLEnableProfile(vertexProfile); cgGLEnableProfile(fragmentProfile);
}


//deactivate
void TradShaderPoint::deactivate() {
    //Deactivate traditional shading
    cgGLDisableProfile(vertexProfile); cgGLDisableProfile(fragmentProfile);
    cgGLDisableTextureParameter(cg_TboLightPos);
    cgGLDisableTextureParameter(cg_TexColor);
}


//enableColorTex
void TradShaderPoint::enableTexColor(GLuint texColor) {
    cgGLSetTextureParameter(cg_TexColor, texColor);
    cgGLEnableTextureParameter(cg_TexColor);
}


//disableTexColor
void TradShaderPoint::disableTexColor() {
    cgGLDisableTextureParameter(cg_TexColor);
}


//enableAlphaReject
void TradShaderPoint::enableAlphaReject(GLfloat alphaReject) {
    glAlphaFunc(GL_GEQUAL, alphaReject);
    glEnable(GL_ALPHA_TEST);
}


//disableAlphaReject
void TradShaderPoint::disableAlphaReject() {
    glDisable(GL_ALPHA_TEST);
}


//setNumLights
void TradShaderPoint::setNumLights(GLushort numLights_) {
    numLights = numLights_;
    cgSetParameter1i(cg_NumLights, numLights);
}
