#ifndef TRAD_SHADER_POINT_H
#define TRAD_SHADER_POINT_H
/* Base class for traditional shading with point lights */


#include "ShadingState.h"
#include "ShaderProgFactory.h"
class Geometry;
class TransformFeedback;


class TradShaderPoint: public ShadingState {
    public:
        ShaderProgFactory* shaderProgFactory;
        TransformFeedback* transformFeedback;
        //Light data
        Geometry* geoLights;
        GLuint tboLightPos;  //TBO with light positions
        CGparameter cg_NumLights, cg_TboLightPos;
        //Object-space to eye-space
        CGprogram vp_LightsObjToEye, vp_ObjToEye;
        //Traditional shading
        CGprogram fp_TradShader;
        CGparameter cg_Material, cg_TexColor;  //Color texture of geometry


    public:
        //Constructor
        TradShaderPoint(GLushort maxLights, Geometry* geoLights_, std::string fileName, std::string stateInfo_);

        //Transform lights' object-space position to eye-space
        void lightsObjToEye();

        //Activate traditional shading fragment program
        void activate();

        //Deactivate traditional shading fragment program
        void deactivate();

        //Enable color texture of geometry to be drawn
        void enableTexColor(GLuint texColor);

        //Disable color texture of geometry
        void disableTexColor();

        //Reject fragments with an alpha value lower than parameter alphaReject
        void enableAlphaReject(GLfloat alphaReject);

        //Stop rejecting fragments with a specific alpha value
        void disableAlphaReject();

        //Set the number of lights to be used
        void setNumLights(GLushort numLights_);
};

#endif
