#include <cstdlib>
#include "TransformFeedback.h"


//Constructor
TransformFeedback::TransformFeedback(GLuint numVertices_, GLsizei numAttribs_, GLenum usage_) {
    usage             = usage_;
    numVertices       = numVertices_;
    propertiesAttribs = new GLint[numAttribs_*3];
    outputBuffers     = new GLuint[numAttribs_];
    glGenBuffers(numAttribs_, outputBuffers);
    numAttribs = 0;
}


//Destructor
TransformFeedback::~TransformFeedback() {
    glDeleteBuffers(numAttribs, outputBuffers);
    delete[] propertiesAttribs;
    delete[] outputBuffers;
}


//enablePositions
void TransformFeedback::enablePositions(GLuint numComponents) {
    propertiesAttribs[3*numAttribs]   = GL_POSITION;
    propertiesAttribs[3*numAttribs+1] = numComponents;
    propertiesAttribs[3*numAttribs+2] = 0;
    positionBuffer = &outputBuffers[numAttribs];

    //Set size and usage
    glBindBuffer(GL_ARRAY_BUFFER, outputBuffers[numAttribs]);
    glBufferData(GL_ARRAY_BUFFER, numVertices*numComponents*sizeof(GLfloat), NULL, usage);

    ++numAttribs;
}


//enableColors
void TransformFeedback::enableColors(GLuint numComponents) {
    propertiesAttribs[3*numAttribs]   = GL_PRIMARY_COLOR;
    propertiesAttribs[3*numAttribs+1] = numComponents;
    propertiesAttribs[3*numAttribs+2] = 0;
    colorBuffer = &outputBuffers[numAttribs];

    //Set size and usage
    glBindBuffer(GL_ARRAY_BUFFER, outputBuffers[numAttribs]);
    glBufferData(GL_ARRAY_BUFFER, numVertices*numComponents*sizeof(GLfloat), NULL, usage);

    ++numAttribs;
}


//enableTexCoords
void TransformFeedback::enableTexCoords(GLuint numComponents) {
    propertiesAttribs[3*numAttribs]   = GL_TEXTURE_COORD_NV;
    propertiesAttribs[3*numAttribs+1] = numComponents;
    propertiesAttribs[3*numAttribs+2] = 0;
    texCoordBuffer = &outputBuffers[numAttribs];

    //Set size and usage
    glBindBuffer(GL_ARRAY_BUFFER, outputBuffers[numAttribs]);
    glBufferData(GL_ARRAY_BUFFER, numVertices*numComponents*sizeof(GLfloat), NULL, usage);

    ++numAttribs;
}


//bind
void TransformFeedback::bind(GLenum mode) {
    glEnable(GL_RASTERIZER_DISCARD_NV);  //No need for generating fragments

    //Specify which attribute belongs to which buffer
    for (GLsizei index = 0; index < numAttribs; ++index)
        glBindBufferBaseNV(GL_TRANSFORM_FEEDBACK_BUFFER_NV, index, outputBuffers[index]);
    glTransformFeedbackAttribsNV(numAttribs, propertiesAttribs, GL_SEPARATE_ATTRIBS_NV);  //Output to separate buffers

    glBeginTransformFeedbackNV(mode);
}


//unbind
void TransformFeedback::unbind() {
    glEndTransformFeedbackNV();
    glDisable(GL_RASTERIZER_DISCARD_NV);
}


//swapBuffers
void TransformFeedback::swapBuffers(GLuint* buffer1, GLuint* buffer2) {
    GLuint temp = *buffer1;
    *buffer1 = *buffer2;
    *buffer2 = temp;
}
