#ifndef TRANSFORM_FEEDBACK_H
#define TRANSFORM_FEEDBACK_H
/* Wrapper class for OpenGL's transform feedback */


#include <GL/glew.h>


class TransformFeedback {
    public:
        GLuint* positionBuffer, * colorBuffer, * texCoordBuffer;


    private:
        GLint* propertiesAttribs;
        GLuint* outputBuffers;
        GLuint numVertices;
        GLenum usage;
        GLsizei numAttribs;  //Number of attributes we output. Initially zero


    public:
        //Constructor. Parameter 'usage_' denotes where and how the data is being used
        TransformFeedback(GLuint numVertices, GLsizei numAttribs_, GLenum usage_);

        //Destructor
        ~TransformFeedback();

        //Enable the rendering of positions to output buffer. Parameter 'numComponents' specifies number of components to render
        void enablePositions(GLuint numComponents);

        //Enable the rendering of colors to output buffer. Parameter 'numComponents' specifies number of components to render
        void enableColors(GLuint numComponents);

        //Enable the rendering of texture coordinates to output buffer. Parameter 'numComponents' specifies number of components to render
        void enableTexCoords(GLuint numComponents);

        //Render to output buffers from now on. Parameter 'mode' denotes what primitive is rendered
        void bind(GLenum mode);

        //Stop rendering to output buffers
        void unbind();

        //Swap buffers
        void swapBuffers(GLuint* buffer1, GLuint* buffer2);
};

#endif
